/**
  ******************************************************************************
  * @file           : main.c
  * @author         : sinomcu.com-AE
  * @brief          : Main program body
  *    
  *          This file provides example code for UART RX and TX swap,Auto BuadRate
  * and printf by uart:UART RX interrupt,Start $/#/%,END 0x0A;Buff len 64;send back
  * recv frame if it is currect.
  * For details, see "readme.txt"    
  *         
  ******************************************************************************
   * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */  
  
#include "system_define.h"

/* LED blink half cycle*/
#define LED_BLINK_ERROR           1         // unit:100ms
#define LED_BLINK_SLOWE           2         // unit:100ms
#define LED_BLINK_NOMAL           5         // unit:100ms
#define COUNT_PERIOD              1000      // count period in ms

#define ABR_TRY_MAX_NUM           64

UartRecvStatus recv_status = WAIT_STAR;
UartRecvFinishFlag recv_finsh = NO_FINISH;

uint8_t uart_buff[UART_BUFF_LEN]={0};

/**
  * @brief  Main program
  * @param  None
  * @retval None
  */
int main(void) {
  uint32_t count_LED = 0,run_count = 0;
  uint32_t led_blink_period = LED_BLINK_NOMAL;

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  sysclk_init();                            // sysclk 48M,HSI 8M /2*12;if not call this function,default HSI 8M
#endif
  SystemCoreClockUpdate();
  
  sys_tick_init();
  gpio_init();

#if(defined(SYSCLK_USE_PLL_48M) && SYSCLK_USE_PLL_48M) 
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_SYSCLK);    // uart clk sysclk ,TX DMA uart 
  usart1_uart_init(UART_BAUD_RATE_VALUE);                            // AN2102,the other side 2 STOP
#else
  MS32_RCC_SetUSARTClockSource(MS32_RCC_USART1_CLKSOURCE_HSI);       // uart clk HSI 8M   
  usart1_uart_init(UART_BAUD_RATE_VALUE);                            // AN2102,the other side 2 STOP  
#endif

  LED1_ON();   
  printf("\r\n\r\n*****USART RX IT Auto BuadRate Example*****");
  printf("\r\n--Inf:System Core Clock %d Hz.",SystemCoreClock);
#if(defined(USART_PIN_SWAP) && USART_PIN_SWAP)  
  printf("\r\n--Inf:Uart Rx Tx Pin Swap.");
#endif
    
#if (ATUO_BAUD_RATE == 0)         // Disable Auto BaudRate
  printf("\r\n--Inf:Uart Auto BaudRate Disalbe.");
#elif (ATUO_BAUD_RATE == 1)       // Auto BaudRate Mode 0
  printf("\r\n--Inf:Uart Auto BaudRate MODE 0.");
  MS32_USART_SetAutoBaudRateMode(USART1, MS32_USART_AUTOBAUD_DETECT_ON_STARTBIT); 
  MS32_USART_EnableAutoBaudRate(USART1);  
#elif (ATUO_BAUD_RATE == 2)      // Auto BaudRate Mode 1
  printf("\r\n--Inf:Uart Auto BaudRate MODE1.");
  MS32_USART_SetAutoBaudRateMode(USART1, MS32_USART_AUTOBAUD_DETECT_ON_FALLINGEDGE); 
  MS32_USART_EnableAutoBaudRate(USART1);
#else
#error "USART Auto Baud Rate Mode Select Error!"
/* MS32_USART_AUTOBAUD_DETECT_ON_7F_FRAME,MS32_USART_AUTOBAUD_DETECT_ON_55_FRAME */
/* ABRE set */
#endif

  printf("\r\n--Inf:Data frame start char'%c'.",UART_START_CHAR); 

#if( (ATUO_BAUD_RATE == 1) || (ATUO_BAUD_RATE == 2) )
  uart_AutobaduRate();
#endif

  Start_100MS_SF_Timer(); 
  Start_User_MS_SF_Timer(COUNT_PERIOD);
  while (1) {
    if(recv_status == UART_ERROR_ORE) {                                                  // uart error interrupt occur
      recv_status =UART_ERROR_CLR;
      led_blink_period = LED_BLINK_SLOWE;
      printf("\r\n##Err:UART ORE Occur!");
    }
    else if((recv_status == UART_ERROR_NIOSE)||(recv_status == UART_ERROR_FE)) {         // uart error interrupt occur
      recv_status =UART_ERROR_CLR;
      led_blink_period = LED_BLINK_ERROR;
      if(recv_status == UART_ERROR_NIOSE)
        printf("\r\n##Err:UART NIOSE ERROR!");
      if(recv_status == UART_ERROR_FE)
        printf("\r\n##Err:UART FE ERROR!");
    }
    else if(recv_finsh == RECV_ERROR) {                                                  // frame format not corrent
      recv_finsh =RECV_ERROR_CLR;
      led_blink_period = LED_BLINK_SLOWE;
      printf("\r\n--###recv char not the correct frame format!");
    }
    else if(recv_finsh == FINISH) {
      recv_finsh = FINISH_CLR;
      led_blink_period =LED_BLINK_NOMAL;
      Send_buff_Uart(uart_buff);                                                         // recv one frame,send back
    }

    if(Get_State_User_MS_SF_Timer() == SF_TIMER_OV) {
      Start_User_MS_SF_Timer(COUNT_PERIOD);   
      run_count++;
      printf("\r\n--Inf: running count:%d ",run_count);
    }

    if(Get_State_100MS_SF_Timer() == SF_TIMER_OV) {
      Start_100MS_SF_Timer(); 
      
      count_LED++;
      if(count_LED >= led_blink_period) {
        LED1_TOGGLE();
        count_LED =0;
      }
    }    
  }
}

/**
  * @brief  uart auto baud rate
  * @param  None
  * @retval ErrorStatus:SUCCESS,ERROR 
  */
ErrorStatus uart_AutobaduRate(void) {
  ErrorStatus fun_status_flag= ERROR;
  FlagStatus ARB_GO_ON_Flag=SET;
  uint8_t ABR_try_times =0,received_char =0;
  uint32_t count_LED = 0,led_blink_period = LED_BLINK_SLOWE;
  uint32_t usart_clk_freq=0,usart_baudrate=0; 

  Start_100MS_SF_Timer(); 
  usart_clk_freq =MS32_RCC_GetUSARTClockFreq(MS32_RCC_USART1_CLKSOURCE);

  MS32_USART_DisableIT_RXNE(USART1);
  while(ARB_GO_ON_Flag ==SET) {
    if(MS32_USART_IsActiveFlag_ABRE(USART1)) {
      printf("\r\n##Err:ABRE Set! Request again.");
      MS32_USART_RequestAutoBaudRate(USART1);    // clear ABRE
    }

    if(MS32_USART_IsActiveFlag_ABR(USART1)) {    // if ABRF set, stop ABR
      printf("\r\n--Inf:ABR Finish.");
      received_char = MS32_USART_ReceiveData8(USART1);      
      if( received_char == UART_START_CHAR) {
        usart_baudrate =MS32_USART_GetBaudRate(USART1, usart_clk_freq, MS32_USART_OVERSAMPLING_16);  
        printf("\r\n--Inf:ABR OK,BaudRate = %d .",usart_baudrate);
        ARB_GO_ON_Flag=RESET;
        fun_status_flag= SUCCESS;
      } else {
        printf("\r\n!!Warn:ARB again.");     
        MS32_USART_RequestAutoBaudRate(USART1);    // clear ABRF  ,restart ABR;
        ABR_try_times++;
      }	
    
      if(recv_status == UART_ERROR_ORE) {                                                  // uart error interrupt occur
        recv_status =UART_ERROR_CLR;
      }

    }
    if(ABR_try_times ==ABR_TRY_MAX_NUM) {
      ARB_GO_ON_Flag=RESET;
      MS32_USART_DisableAutoBaudRate(USART1);  
      MS32_USART_Disable(USART1);  
      usart1_uart_init(UART_BAUD_RATE_VALUE);  
      printf("\r\n!!Warn:ARB try too many times.STOP ABR,BuadRate default %d",UART_BAUD_RATE_VALUE); 
    }

    if(Get_State_100MS_SF_Timer() == SF_TIMER_OV) {
      Start_100MS_SF_Timer(); 
      
      count_LED++;
      if(count_LED >= led_blink_period) {
        LED2_TOGGLE();
        count_LED =0;
      }
    }
  }  //end while(ARB_GO_ON_Flag ==SET)
  MS32_USART_EnableIT_RXNE(USART1); 
  LED2_OFF();
  return fun_status_flag;
}

/**
  * @brief  SystemClock_Config HSI/2 to PLL*12 sysclk 48M
  * @param  None
  * @retval None
  */
void sysclk_init(void) {
  MS32_RCC_OscInitTypeDef osc_cfg;
  MS32_RCC_PllInitTypeDef pll_cfg;
  MS32_RCC_ClkInitTypeDef clk_cfg;

  // get and clear reset flag 
  MS32_RCC_GetResetFlag(ENABLE);
  
  // osc config 
  osc_cfg.HSIState = MS32_RCC_HSI_ON;
  osc_cfg.HSEState = MS32_RCC_HSE_OFF;
  osc_cfg.LSIState = MS32_RCC_LSI_OFF;
  osc_cfg.LSEState = MS32_RCC_LSE_OFF;
  MS32_RCC_OscConfig(&osc_cfg);
  
  // pll config 
  pll_cfg.PLLState  = MS32_RCC_PLL_ON;
  pll_cfg.PLLSource = MS32_RCC_PLL_SOURCE_HSI_DIV_2;       // HSI/2 4M
  pll_cfg.PLLMUL    = MS32_RCC_PLL_MUL12;
  pll_cfg.PREDIV    = MS32_RCC_PLL_PREDIV_DIV1;
  MS32_RCC_PllConfig(&pll_cfg);
  
  // sysclk config 
  clk_cfg.SYSCLKSource   = MS32_RCC_SYSCLK_SOURCE_PLL;
  clk_cfg.AHBCLKDivider  = MS32_RCC_SYSCLK_DIV1;
  clk_cfg.APB1CLKDivider = MS32_RCC_HCLK_DIV1;
  MS32_RCC_ClockConfig(&clk_cfg);
}

/******************************** END OF FILE *********************************/
