/**
  ******************************************************************************
  * @file    user_config.h
  * @author  sinomcu.com-AE
  * @brief   Header file of user config.
  * 
  ******************************************************************************
  * @attention 
  *                           DISCLAIMER
  * The present firmware which is for guidance only aims at providing customers
  * with coding information regarding their products in order for them to save
  * time. As a result, sinomcu.com shall not be held liable for any direct,indirect
  * or consequential damages with respect to any claims arising from the content 
  * of such firmware and/or the use made by customers of the coding information 
  * contained herein in connection with their products.
  *
  * <h2><center>&copy; Copyright (c) 2021 sinomcu.com All rights reserved.</center></h2>
  *
  ******************************************************************************
  */
// <<< Use Configuration Wizard in Context Menu >>>
#ifndef __USER_CONFIG_H
#define __USER_CONFIG_H

// <c1> SystemClock 48MHz
// <i> System Clock use PLL out 48M
#define SYSCLK_USE_PLL_48M      1         // 1:sysclk PLL48M,0 or not define:HSI 8M 
// </c>

// <o> Auto Baud Rate Mode
// <i> select auto uart baud rate mode
// <i> Mode0 MSX--LSB:xxxxxxx1;eg.#(0x23)
// <i> Mode1 MSX--LSB:xxxxxx01;eg.%(0x25)
// <i> Mode2 LSB first:0x7F,MS32F031 not support
// <i> Mode3 LSB first:0x55,MS32F031 not support
// <0=>    Disable Auto BaudRate
// <1=>    Auto BaudRate Mode 0
// <2=>    Auto BaudRate Mode 1
/* <3=>    Auto BaudRate Mode 2 */
/* <4=>    Auto BaudRate Mode 3 */
#define ATUO_BAUD_RATE            1         

// <c1> Tx/Rx Pin Swap
// <i> USART Tx Rx pin swap
//#define USART_PIN_SWAP            1         // 1:Retargets printf to USART,0 or not define:printf null out
// </c>

// <o> Uart Default Baud Rate
// <i> select uart default baud rate
// <0=>    2400
// <1=>    4800
// <2=>    9600
// <3=>    38400
// <4=>    115200
#define UART_BAUD_RATE            4         

#if (UART_BAUD_RATE == 4) 
#define UART_BAUD_RATE_VALUE      115200 
#elif (UART_BAUD_RATE == 3) 
#define UART_BAUD_RATE_VALUE      38400
#elif (UART_BAUD_RATE == 2) 
#define UART_BAUD_RATE_VALUE      9600
#elif (UART_BAUD_RATE == 1) 
#define UART_BAUD_RATE_VALUE      4800
#else
#define UART_BAUD_RATE_VALUE      2400
#endif

// <c1> Printf to USART
// <i> Retargets printf to USART
#define EN_PRINT_UART             1         // 1:Retargets printf to USART,0 or not define:printf null out
// </c>

#endif /* __USER_CONFIG_H */
// <<< end of configuration section >>>
/******************************** END OF FILE *********************************/
